@@ -212,6 +212,12 @@ void applyStateHelper<MSAAEnabled>(const MSAAEnabled *state, SubmissionContext *
     gc->setMSAAEnabled(std::get<0>(state->values()));
 }
 
+template<>
+void applyStateHelper<DepthRange>(const DepthRange *state, SubmissionContext *gc)
+{
+    const auto values = state->values();
+    gc->depthRange(std::get<0>(values), std::get<1>(values));
+}
 
 template<>
 void applyStateHelper<DepthTest>(const DepthTest *state, SubmissionContext *gc)
@@ -219,6 +225,11 @@ void applyStateHelper<DepthTest>(const DepthTest *state, SubmissionContext *gc)
     gc->depthTest(std::get<0>(state->values()));
 }
 
+template<>
+void applyStateHelper<RasterMode>(const RasterMode *state, SubmissionContext *gc)
+{
+    gc->rasterMode(std::get<0>(state->values()), std::get<1>(state->values()));
+}
 
 template<>
 void applyStateHelper<NoDepthMask>(const NoDepthMask *state, SubmissionContext *gc)
@@ -226,7 +237,6 @@ void applyStateHelper<NoDepthMask>(const NoDepthMask *state, SubmissionContext *
     gc->depthMask(std::get<0>(state->values()));
 }
 
-
 template<>
 void applyStateHelper<CullFace>(const CullFace *state, SubmissionContext *gc)
 {
@@ -379,6 +389,7 @@ void SubmissionContext::initialize()
 {
     GraphicsContext::initialize();
     m_textureContext.initialize(this);
+    m_imageContext.initialize(this);
 }
 
 void SubmissionContext::resolveRenderTargetFormat()
@@ -469,6 +480,7 @@ void SubmissionContext::endDrawing(bool swapBuffers)
     if (m_ownCurrent)
         m_gl->doneCurrent();
     m_textureContext.endDrawing();
+    m_imageContext.endDrawing();
 }
 
 void SubmissionContext::activateRenderTarget(Qt3DCore::QNodeId renderTargetNodeId, const AttachmentPack &attachments, GLuint defaultFboId)
@@ -691,8 +703,8 @@ QImage SubmissionContext::readFramebuffer(const QRect &rect)
     GLint samples = 0;
     m_gl->functions()->glGetIntegerv(GL_SAMPLES, &samples);
     if (samples > 0 && !m_glHelper->supportsFeature(GraphicsHelperInterface::BlitFramebuffer)) {
-        qWarning () << Q_FUNC_INFO << "Unable to capture multisampled framebuffer; "
-                       "Required feature BlitFramebuffer is missing.";
+        qCWarning(Backend) << Q_FUNC_INFO << "Unable to capture multisampled framebuffer; "
+                                             "Required feature BlitFramebuffer is missing.";
         return img;
     }
 
@@ -715,7 +727,7 @@ QImage SubmissionContext::readFramebuffer(const QRect &rect)
         if (status != GL_FRAMEBUFFER_COMPLETE) {
             gl->glDeleteRenderbuffers(1, &rb);
             gl->glDeleteFramebuffers(1, &fbo);
-            qWarning () << Q_FUNC_INFO << "Copy-framebuffer not complete: " << status;
+            qCWarning(Backend) << Q_FUNC_INFO << "Copy-framebuffer not complete: " << status;
             return img;
         }
 
@@ -801,7 +813,7 @@ bool SubmissionContext::activateShader(ProgramDNA shaderDNA)
             m_activeShaderDNA = shaderDNA;
         } else {
             m_glHelper->useProgram(0);
-            qWarning() << "No shader program found for DNA";
+            qCWarning(Backend) << "No shader program found for DNA";
             m_activeShaderDNA = 0;
             return false;
         }
@@ -859,7 +871,7 @@ void SubmissionContext::activateDrawBuffers(const AttachmentPack &attachments)
             }
         }
     } else {
-        qWarning() << "FBO incomplete";
+        qCWarning(Backend) << "FBO incomplete";
     }
 }
 
@@ -870,6 +882,7 @@ void SubmissionContext::setActiveMaterial(Material *rmat)
         return;
 
     m_textureContext.deactivateTexturesWithScope(TextureSubmissionContext::TextureScopeMaterial);
+    m_imageContext.deactivateImages();
     m_material = rmat;
 }
 
@@ -927,6 +940,16 @@ void SubmissionContext::applyState(const StateVariant &stateVariant)
         break;
     }
 
+    case DepthRangeMask: {
+        applyStateHelper<DepthRange>(static_cast<const DepthRange *>(stateVariant.constState()), this);
+        break;
+    }
+
+    case RasterModeMask: {
+        applyStateHelper<RasterMode>(static_cast<const RasterMode *>(stateVariant.constState()), this);
+        break;
+    }
+
     case FrontFaceStateMask: {
         applyStateHelper<FrontFace>(static_cast<const FrontFace *>(stateVariant.constState()), this);
         break;
@@ -1009,6 +1032,9 @@ void SubmissionContext::resetMasked(qint64 maskOfStatesToReset)
     if (maskOfStatesToReset & StencilTestStateMask)
         funcs->glDisable(GL_STENCIL_TEST);
 
+    if (maskOfStatesToReset & DepthRangeMask)
+        depthRange(0.0f, 1.0f);
+
     if (maskOfStatesToReset & DepthTestStateMask)
         funcs->glDisable(GL_DEPTH_TEST);
 
@@ -1050,6 +1076,11 @@ void SubmissionContext::resetMasked(qint64 maskOfStatesToReset)
 
     if (maskOfStatesToReset & LineWidthMask)
         funcs->glLineWidth(1.0f);
+
+#ifndef QT_OPENGL_ES_2
+    if (maskOfStatesToReset & RasterModeMask)
+        m_glHelper->rasterMode(GL_FRONT_AND_BACK, GL_FILL);
+#endif
 }
 
 void SubmissionContext::applyStateSet(RenderStateSet *ss)
@@ -1104,6 +1135,33 @@ void SubmissionContext::clearStencilValue(int stencil)
     }
 }
 
+GLFence SubmissionContext::fenceSync()
+{
+    return m_glHelper->fenceSync();
+}
+
+void SubmissionContext::clientWaitSync(GLFence sync, GLuint64 nanoSecTimeout)
+{
+    qDebug() << Q_FUNC_INFO << sync;
+    m_glHelper->clientWaitSync(sync, nanoSecTimeout);
+}
+
+void SubmissionContext::waitSync(GLFence sync)
+{
+    qDebug() << Q_FUNC_INFO << sync;
+    m_glHelper->waitSync(sync);
+}
+
+bool SubmissionContext::wasSyncSignaled(GLFence sync)
+{
+    return m_glHelper->wasSyncSignaled(sync);
+}
+
+void SubmissionContext::deleteSync(GLFence sync)
+{
+    m_glHelper->deleteSync(sync);
+}
+
 void SubmissionContext::setUpdatedTexture(const Qt3DCore::QNodeIdVector &updatedTextureIds)
 {
     m_updateTextureIds = updatedTextureIds;
@@ -1126,20 +1184,50 @@ bool SubmissionContext::setParameters(ShaderParameterPack &parameterPack)
     // Update the uniforms with the correct texture unit id's
     PackUniformHash &uniformValues = parameterPack.uniforms();
 
+    // Fill Texture Uniform Value with proper texture units
+    // so that they can be applied as regular uniforms in a second step
     for (int i = 0; i < parameterPack.textures().size(); ++i) {
-        const ShaderParameterPack::NamedTexture &namedTex = parameterPack.textures().at(i);
+        const ShaderParameterPack::NamedResource &namedTex = parameterPack.textures().at(i);
         // Given a Texture QNodeId, we retrieve the associated shared GLTexture
         if (uniformValues.contains(namedTex.glslNameId)) {
-            GLTexture *t = manager->glTextureManager()->lookupResource(namedTex.texId);
+            GLTexture *t = manager->glTextureManager()->lookupResource(namedTex.nodeId);
             if (t != nullptr) {
-                UniformValue &texUniform = uniformValues[namedTex.glslNameId];
+                UniformValue &texUniform = uniformValues.value(namedTex.glslNameId);
                 if (texUniform.valueType() == UniformValue::TextureValue) {
-                    const int texUnit = m_textureContext.activateTexture(TextureSubmissionContext::TextureScopeMaterial, t);
+                    const int texUnit = m_textureContext.activateTexture(TextureSubmissionContext::TextureScopeMaterial, m_gl, t);
                     texUniform.data<int>()[namedTex.uniformArrayIndex] = texUnit;
                     if (texUnit == -1) {
                         if (namedTex.glslNameId != irradianceId &&
                             namedTex.glslNameId != specularId) {
                             // Only return false if we are not dealing with env light textures
+                            qCWarning(Backend) << "Unable to find suitable Texture Unit";
+                            return false;
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+    // Fill Image Uniform Value with proper image units
+    // so that they can be applied as regular uniforms in a second step
+    for (int i = 0; i < parameterPack.images().size(); ++i) {
+        const ShaderParameterPack::NamedResource &namedTex = parameterPack.images().at(i);
+        // Given a Texture QNodeId, we retrieve the associated shared GLTexture
+        if (uniformValues.contains(namedTex.glslNameId)) {
+            ShaderImage *img = manager->shaderImageManager()->lookupResource(namedTex.nodeId);
+            if (img != nullptr) {
+                GLTexture *t = manager->glTextureManager()->lookupResource(img->textureId());
+                if (t == nullptr) {
+                    qCWarning(Backend) << "Shader Image referencing invalid texture";
+                    continue;
+                } else {
+                    UniformValue &imgUniform = uniformValues.value(namedTex.glslNameId);
+                    if (imgUniform.valueType() == UniformValue::ShaderImageValue) {
+                        const int imgUnit = m_imageContext.activateImage(img, t);
+                        imgUniform.data<int>()[namedTex.uniformArrayIndex] = imgUnit;
+                        if (imgUnit == -1) {
+                            qCWarning(Backend) << "Unable to bind Image to Texture";
                             return false;
                         }
                     }
@@ -1193,10 +1281,12 @@ bool SubmissionContext::setParameters(ShaderParameterPack &parameterPack)
     for (const ShaderUniform &uniform : activeUniforms) {
         // We can use [] as we are sure the the uniform wouldn't
         // be un activeUniforms if there wasn't a matching value
-        const UniformValue &v = values[uniform.m_nameId];
+        const UniformValue &v = values.value(uniform.m_nameId);
 
-        // skip invalid textures
-        if (v.valueType() == UniformValue::TextureValue && *v.constData<int>() == -1)
+        // skip invalid textures/images
+        if ((v.valueType() == UniformValue::TextureValue ||
+             v.valueType() == UniformValue::ShaderImageValue) &&
+            *v.constData<int>() == -1)
             continue;
 
         applyUniform(uniform, v);
